@@ -37,9 +37,9 @@ Here are the ways you can hook into Phoebe code:
 C<@extensions> is a list of code references allowing you to handle additional
 URLs; return 1 if you handle a URL; each code reference gets called with $stream
 (L<Mojo::IOLoop::Stream>), the first line of the request (a Gemini URL, a Gopher
-selector, a finger user, a HTTP request line), a hash reference (with the
-headers of HTTP requests or the parameters of Titan requests), a buffer of bytes
-(e.g. for Titan or HTTP PUT or POST requests), and (sometimes) size.
+selector, a finger user, a HTTP request line), a hash reference for the headers
+(in the case of HTTP requests), and a buffer of bytes (e.g. for Titan or HTTP
+PUT or POST requests).
 
 C<@main_menu> adds more lines to the main menu, possibly links that aren't
 simply links to existing pages.
@@ -139,7 +139,7 @@ for it, for Gemini only:
 
 package App::Phoebe;
 use Modern::Perl '2018';
-use File::Slurper qw(read_text read_binary read_dir write_text write_binary);
+use File::Slurper qw(read_text read_binary read_lines read_dir write_text write_binary);
 use Encode qw(encode_utf8 decode_utf8);
 use Net::IDN::Encode qw(domain_to_ascii);
 use Socket qw(:addrinfo SOCK_RAW);
@@ -151,7 +151,7 @@ use Mojo::IOLoop;
 use Mojo::Log;
 use utf8;
 
-our $VERSION = 4.05;
+our $VERSION = 4.02;
 
 require Exporter;
 our @ISA = qw(Exporter);
@@ -160,12 +160,12 @@ our @EXPORT_OK = qw(@extensions @main_menu @footer $log $server $full_url_regex
 		    result get_ip_numbers run_extensions pages blog blog_pages
 		    text save_page serve_index serve_page serve_raw serve_html
 		    serve_history serve_diff html_page to_html @request_handlers
-		    handle_request process_titan process_gemini valid_params
-		    valid_id valid_mime_type valid_size valid_token print_link
-		    all_logs gemini_link colourize modified changes diff
-		    bogus_hash quote_html write_page @known_fingerprints
-		    with_lock wiki_dir to_url handle_titan footer atom rss files
-		    space_links search decode_query);
+		    handle_request process_titan process_gemini valid_id
+		    valid_mime_type valid_size valid_token print_link all_logs
+		    gemini_link colourize modified changes diff bogus_hash
+		    quote_html write_page @known_fingerprints with_lock wiki_dir
+		    to_url handle_titan footer atom rss files space_links search
+		    decode_query);
 
 # Phoebe variables you can set in the config file
 
@@ -217,8 +217,7 @@ sub get_ip_numbers {
 # options.
 sub host_regex {
   my $stream = shift;
-  my $re = join("|", map { quotemeta domain_to_ascii $_ } keys %{$server->{host}});
-  return qr($re)i; # case insensitive hostnames
+  return join("|", map { quotemeta domain_to_ascii $_ } keys %{$server->{host}});
 }
 
 # A regular expression matching wiki spaces in URLs. The tricky part is that we
@@ -294,16 +293,10 @@ sub process_titan {
   eval {
     local $SIG{'ALRM'} = sub { $log->error("Timeout processing upload $request") };
     alarm(10); # timeout
-    if (run_extensions($stream, $request, $upload, $buffer, $size)) {
-      # config file goes first
-    } else {
-      save_page($stream, $upload->{host}, $upload->{space}, $upload->{id},
-		$upload->{params}->{mime}, $buffer, $size);
-    }
+    save_page($stream, $upload->{host}, $upload->{space}, $upload->{id},
+	      $upload->{params}->{mime}, $buffer, $size);
     alarm(0);
   };
-  # save page might still be waiting for the lock so we must not close the
-  # stream: save_page will close the stream
   return unless $@;
   $log->error("Error: $@");
   $stream->close_gracefully();
@@ -314,38 +307,19 @@ sub save_page {
   my $host = shift;
   my $space = shift;
   my $id = shift;
-  my $type = shift || "text/plain";
+  my $type = shift;
   my $data = shift;
   my $length = shift;
-  # If the operation succeeds, we can close the stream; if the operation fails,
-  # we can close the stream; but if the operation was rescheduled, we must not
-  # close the stream!
-  if ($type ne "text/plain" and $type ne "text/gemini") {
+  if ($type ne "text/plain") {
     if ($length == 0) {
-      with_lock($stream, $host, $space,
-		sub {
-		  delete_file($stream, $host, $space, $id);
-		  $stream->close_gracefully();
-		});
+      with_lock($stream, $host, $space, sub { delete_file($stream, $host, $space, $id) } );
     } else {
-      with_lock($stream, $host, $space,
-		sub {
-		  write_file($stream, $host, $space, $id, $data, $type);
-		  $stream->close_gracefully();
-		});
+      with_lock($stream, $host, $space, sub { write_file($stream, $host, $space, $id, $data, $type) } );
     }
   } elsif ($length == 0) {
-    with_lock($stream, $host, $space,
-	      sub {
-		delete_page($stream, $host, $space, $id);
-		$stream->close_gracefully();
-	      });
+    with_lock($stream, $host, $space, sub { delete_page($stream, $host, $space, $id) } );
   } elsif (utf8::decode($data)) { # decodes in-place and returns success
-    with_lock($stream, $host, $space,
-	      sub {
-		write_page($stream, $host, $space, $id, $data);
-		$stream->close_gracefully();
-	      });
+    with_lock($stream, $host, $space, sub { write_page($stream, $host, $space, $id, $data) } );
   } else {
     $log->debug("The text is invalid UTF-8");
     result($stream, "59", "The text is invalid UTF-8");
@@ -375,11 +349,9 @@ sub with_lock {
     if ($@) {
       $log->error("Unable to run code with locked $lock: $@");
       result($stream, "40", "An error occured, unfortunately");
-      $stream->close_gracefully();
     }
-    # in the successful case, with_lock doesn't close in case there is more code
-    # that needs to run, or possibly $code has closed the stream.
     rmdir($lock);
+    $stream->close_gracefully();
   } elsif ($count > 25) {
     $log->error("Unable to unlock $lock");
     result($stream, "40", "The wiki is locked; try again in a few seconds");
@@ -478,7 +450,7 @@ sub delete_page {
   my $index = "$dir/index";
   if (-f $index) {
     # remove $id from the index
-    my @pages = grep { $_ ne $id } split /\n/, read_text $index;
+    my @pages = grep { $_ ne $id } read_lines $index;
     write_text($index, join("\n", @pages, ""));
   }
   my $changes = "$dir/changes.log";
@@ -696,7 +668,7 @@ sub handle_url {
 # if you call this yourself, $id must look like "page/foo"
 sub to_url {
   my $stream = shift;
-  my $host = lc shift;
+  my $host = shift;
   my $space = shift;
   my $id = shift;
   my $scheme = shift || "gemini";
@@ -755,9 +727,9 @@ sub pages {
     return if not -d "$dir/page";
     my @pages = map { s/\.gmi$//; $_ } read_dir("$dir/page");
     write_text($index, join("\n", @pages, ""));
-    return sort newest_first @pages;
+    return sort { newest_first($stream, $host, $a, $b) } @pages;
   }
-  my @lines = sort newest_first split /\n/, read_text $index;
+  my @lines = sort { newest_first($stream, $host, $a, $b) } read_lines $index;
   return grep /$re/i, @lines if $re;
   return @lines;
 }
@@ -1850,7 +1822,7 @@ sub serve_file {
     result($stream, "40", "Metadata not found");
     return;
   }
-  my %meta = (map { split(/: /, $_, 2) } split /\n/, read_text $meta);
+  my %meta = (map { split(/: /, $_, 2) } read_lines($meta));
   if (not $meta{'content-type'}) {
     result($stream, "59", "Metadata corrupt");
     return;
@@ -2022,7 +1994,7 @@ sub is_upload {
     my $host = $1;
     my($scheme, $authority, $path, $query, $fragment) =
 	$request =~ m|(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|;
-    if ($path =~ m!^(?:/($spaces_regex))?(?:/raw|/page|/file)?/([^/;=&]+(?:;\w+=[^;=&]+)+)!) {
+    if ($path =~ m!^(?:/($spaces_regex))?(?:/raw|/page)?/([^/;=&]+(?:;\w+=[^;=&]+)+)!) {
       my $space = $1;
       my ($id, @params) = split(/[;=&]/, $2);
       my $params = { map {decode_utf8(uri_unescape($_))} @params };
@@ -2107,20 +2079,17 @@ sub valid_mime_type {
   my $space = shift;
   my $id = shift;
   my $params = shift;
-  my $type = $params->{mime} || "text/plain";
+  my $type = $params->{mime};
   my ($main_type) = split(/\//, $type, 1);
   my @types = @{$server->{wiki_mime_type}};
-  # the wiki always allows text/plain or text/gemini
-  if ($type eq "text/plain" or $type eq "text/gemini") {
-    return 1;
-  } elsif (not @types) {
-    $log->debug("This wiki does not allow file uploads");
-    result($stream, "59", "This wiki does not allow file uploads");
+  if (not $type) {
+    $log->debug("Uploads require a MIME type");
+    result($stream, "59", "Uploads require a MIME type");
     $stream->close_gracefully();
     return;
-  } elsif (not grep(/^$type$/, @types) and not grep(/^$main_type$/, @types)) {
+  } elsif ($type ne "text/plain" and not grep(/^$type$/, @types) and not grep(/^$main_type$/, @types)) {
     $log->debug("This wiki does not allow $type");
-    result($stream, "59", "This wiki does not allow $type, only @types");
+    result($stream, "59", "This wiki does not allow $type");
     $stream->close_gracefully();
     return;
   }
